package archetect

import sbt._

trait ArchetectProject extends Project with FileTasks {
  import Process._
  val arcSource = "src" / "arc"
  val arcOutput = outputPath / "arc"
  override def watchPaths = super.watchPaths +++ (arcSource ** "*")
  
  def templateMappings = Map[String, Any]()

  lazy val archetect = dynamic(archetectTasks)

  def archetectTasks = task { None } named("archetect-complete") dependsOn (
    descendents(arcSource ##, "*").get.filter(!_.isDirectory).map { in =>
      val out = Path.fromString(arcOutput, in.relativePath)
      val tmpl = """(.*)\{\{(.*)\}\}(.*\n)""".r
      def template(str: String): String = str match {
        case tmpl(before, key, after) => template(before + templateMappings(key) + after)
        case _ => str
      }
      fileTask(out from in) {
        FileUtilities.readStream(in.asFile, log) { stm =>
          FileUtilities.write(out.asFile, log) { writer =>
            // not using line-by-line because of 2.7 / 2.8 source incompatibility
            writer write template(io.Source.fromInputStream(stm).mkString)
            None
          }
        }
      }
    }.toSeq: _*
  )
}
